#ifndef __tetris_h
#define __tetris_h

#include "music.h"

#define WIPESPEED 50

#define MAXLEVEL  13	/* At maxlevel, the blocks drop without any delay */

/* All the states here must be negative, except S_GAME,      *
 * which is 0. Also, S_MBLOCK must be S_INIT+1 and S_GAME-1. */
 
#define S_MIN    -100	/* Just a marker           */

#define S_PAUSE  -5		/* Pause                   */
#define S_DROP   -4		/* Dropping block          */
#define S_OVER   -3		/* Game over               */
#define S_INIT   -2		/* Just started            */
#define S_MBLOCK -1		/* Need to make new block  */
#define S_GAME    0		/* A block is falling down */

/* Todo: Check this. It may be incorrect. */
#define S_WIPE (24*MAINRATE/WIPESPEED S_MIN)

typedef int tetblock[4][4];
typedef struct
{
	int Field[18][10];
	
	int WipeLines[18];
	int WipeCount;
	
	int Score;
	int Lines;
	int Level;
	
	int NextColor;
	tetblock NextBlock;
	
	int ThisColor;
	tetblock ThisBlock;
	
	int asento, blnum, nextblnum;
	int blockx, blocky;
	
	int State, SavedState;
	int NeedRefresh;
	
	int dispx,dispy,nextx,statx;
	
	int iscomputer;
	int NoHumanOpponents;
	
	struct MusicSave gameovermusicsave;
} tetris_t;


/* Computer player settings */

/* SKY_LIMIT defines the line which above
 * the blocks should never be placed.
 */ 
#define SKY_LIMIT	0
#define SKY_PUNISHMENT	-100

/* SKY_BONUS is the multiplication factor
 * which is used if SKY_LIMIT is not reached.
 * It can as well be 0, but not negative.
 * Very big values should not be used, because
 * too big values override the other bonuses.
 */
#define SKY_BONUS	0

/* Each of these four bonuses should be divisable with 12.
 * If one is set to 0, the corresponding bonus calculation
 * is not done.
 */ 
#define LEFT_BONUS	12
#define RIGHT_BONUS	12
#define DOWN_BONUS	48
#define TOP_BONUS	12


extern void InitTetris(tetris_t *tet,
	int lev,int hc,
	int x,int y,
	int nx,int sx,
	int iscomputer);
extern void KillTetris(tetris_t *tet);
extern void TickTetris(tetris_t *tet, int numticks);

/* Key values for GrantKey() - value 0 not allowed. */
#define KEY_LT		1
#define KEY_RT		2
#define KEY_DOWN	3
#define KEY_TURN1	4
#define KEY_TURN2	5
#define KEY_DROP	6
#define KEY_PAUSE	7

/* Control */
extern void GrantKey(tetris_t *tet, int key);

#endif	/* __tetris_h defined */
